@@ -1,8 +1,9 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
 import Database.Persist.Sqlite
+import Control.Monad.Logger
 
 $(return []) -- just force TH to run
 
 main :: IO ()
-main = withSqliteConn ":memory:" $ const $ return ()
+main = runStderrLoggingT $ withSqliteConn ":memory:" $ const $ return ()
